%% Copyright (C) 2013-2022 Alexander Barth
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; If not, see <http://www.gnu.org/licenses/>.


function [gid,varid] = ncvarid(ncid,varname)

  if strcmp(varname(1), '/')
    i = find(varname == '/',1,'last');
    groupname = varname(1:i-1);
    varname = varname(i+1:end);
    gid = netcdf_inqGrpFullNcid(ncid,groupname);
  else
    gid = ncid;
  endif

  varid = netcdf_inqVarID(gid, varname);

endfunction
